from django.db import models
from account.models import Account  # Importing Account model
from django.utils.translation import gettext_lazy as _

class AssetCategory(models.Model):
    name = models.CharField(max_length=255, null=True, blank=True)

    class Meta:
        verbose_name = "Asset Category"
        verbose_name_plural = "Asset Categories"

    def __str__(self):
        return self.name or "Unnamed Category"


class Asset(models.Model):
    STATUS_CHOICES = [
        ('Available', 'Available'),
        ('Issued', 'Issued'),
        ('Returned', 'Returned'),
        ('Damaged', 'Damaged'),
    ]

    asset_name = models.CharField(max_length=255, null=True, blank=True)
    category = models.ForeignKey(AssetCategory, on_delete=models.SET_NULL, null=True, blank=True)
    serial_number = models.CharField(max_length=255, null=True, blank=True, unique=True)
    value = models.DecimalField(max_digits=10, decimal_places=2, null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    image = models.ImageField(upload_to='asset_images/', null=True, blank=True)
    account = models.ForeignKey(Account, on_delete=models.SET_NULL, null=True, blank=True, related_name='assets')
    return_date = models.DateField(null=True, blank=True)
    date_issued = models.DateField(null=True, blank=True)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='Available')
    return_comments = models.TextField(null=True, blank=True)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name = "Asset"
        verbose_name_plural = "Assets"

    def save(self, *args, **kwargs):
        # Set default status based on the presence of a user (account)
        if not self.pk:  # Only when creating a new object
            if self.account:
                self.status = 'Issued'
            else:
                self.status = 'Available'

        # Set the issued date to today if an account is present and date_issued is not set
        if self.account and not self.date_issued:
            from datetime import date
            self.date_issued = date.today()

        super().save(*args, **kwargs)

    def __str__(self):
        return self.asset_name or "Unnamed Asset"


 

class AssetIssueHistory(models.Model):
    asset = models.ForeignKey(Asset, on_delete=models.CASCADE)
    user_account = models.ForeignKey(Account, on_delete=models.SET_NULL, null=True, blank=True)  # Can be null if no user is selected
    status = models.CharField(max_length=20,default='Issued')
    return_comments = models.TextField(blank=True, null=True)
    date_updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f"History for {self.asset.asset_name} - {self.status} on {self.date_updated}"
